unit BLabel;
interface
uses
  SysUtils, WinTypes, WinProcs, Controls, StdCtrls, Classes, Graphics, Menus;

type
  TLabelStyle = (lsLowered, lsNormal, lsRaised);

  TBLabel = class(TLabel)
  private
    FOffset:     Integer;
    FLabelStyle: TLabelStyle;
    procedure Paint; override;
  protected
    procedure SetLabelStyle(Value: TLabelStyle);
    procedure SetOffset(Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property LabelStyle: TLabelStyle read FLabelStyle
                                     write SetLabelStyle default lsNormal;
    property Offset: Integer read FOffset
                             write SetOffset default 1;
  end;

procedure Register;

implementation

  constructor TBLabel.Create(AOwner: TComponent);
  begin
    inherited Create(AOwner);
    FOffset := 1;
    FLabelStyle := lsNormal;
    Transparent := True
  end;

  procedure TBLabel.SetLabelStyle(Value: TLabelStyle);
  begin
    if FLabelStyle <> Value then
    begin
      FLabelStyle := Value;
      Invalidate
    end
  end;

  procedure TBLabel.SetOffset(Value: Integer);
  begin
    if FOffset <> Value then
    begin
      FOffset := Value;
      Invalidate
    end
  end;

  procedure TBLabel.Paint;
  const
    { Array that encapsulates some of WinProcs.DrawText's flags }
    Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  var Text: array[0..255] of Char;
      CalcOffset: Integer;
      OldColor: TColor;
      TempRect: TRect;
  begin
    CalcOffset := (Ord(FLabelStyle)-1) * FOffset;
    if (CalcOffset <> 0) then
    begin
      Canvas.Brush.Style := bsClear;   { set clear background }
      Canvas.Font := Self.Font;   { insure Canvas Font is set }
      OldColor := Font.Color;             { save font's color }
      GetTextBuf(Text, SizeOf(Text));   { Get Label's Caption }

      Canvas.Font.Color := clWhite;  { make font white }
      { Create a rect that is offset for the top bevel }
      TempRect := Rect(ClientRect.Left - CalcOffset,
                       ClientRect.Top - CalcOffset,
                       ClientRect.Right - CalcOffset,
                       ClientRect.Bottom - CalcOffset);
      { Draw offset text }
      DrawText(Canvas.Handle, Text, StrLen(Text),
               TempRect, DT_EXPANDTABS or
               DT_WORDBREAK or Alignments[Alignment]);

      Canvas.Font.Color := clBlack;     { make font black }
      { Create rect that is offset for the bottom bevel }
      TempRect := Rect(ClientRect.Left + CalcOffset,
                       ClientRect.Top + CalcOffset,
                       ClientRect.Right + CalcOffset,
                       ClientRect.Bottom + CalcOffset);
      { Draw the offset text }
      DrawText(Canvas.Handle, Text, StrLen(Text),
               TempRect, DT_EXPANDTABS or
               DT_WORDBREAK or Alignments[Alignment]);

      Canvas.Font.Color := OldColor
    end;
    inherited Paint
  end;

  procedure Register;
  begin
    RegisterComponents('Dr.Bob', [TBLabel])
  end;
end.